/* Version 1.11 -- Program SimpleLovera. P. Zeitler, Lehigh University 1 March, 2004

   C translation of Oscar Lovera's program ages.f.
   (translated from original Pascal translation!)
   
   Handles heating as well as cooling as long as, after the model start
   there is complete cooling from well above closure to below closure
   
   Input files are file lovera.in, equivalent to Lovera's ages_me.in, 
   which has cooling history and sample diffusion and domain parameters
   
   and
   
   file goal.in, which gives the observed age spectrum. This file is in
   the same format as that used for the Arvert inversion (fractional loss,
   age, error in age, fit flag). The last two parameters aren't used by this program
   but I figure in  normal use you will be assembling this file anyway. The program
   calculates a heating schedule based on the goal.in losses and the sample's
   kinetic properties. 
   
   Output is in the form of file lovera.out, which is a simple list of 
   the cumulative weighted losses and ages
   
   and
   
   file lovera.spec, which is the same data in age-spectrum format.
   Both of these files also report the goal spectrum as well.
   
   Note that if you are doing theoeretical what-if modeling, you will still need
   a goal.in file, so you will hve to concoct one, remembering that it's the fractional
   losses that are relevant to this program.
   
   25 June, 2004. Version 1.11. Minor tweak to make spec output excel, but age list BBedit.
   
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sioux.h>
#include <time.h>
 #include <assert.h>

	#define    nxi 50001
	#define    nch  1005
	#define    ntp   100
	#define     pi     3.14159265
	#define xlambd     0.000543
	#define      R     0.001987

	typedef double mediumarray[101];
	
	double InterpolateAge(double,double,double,double,double);

void main(void)
{
	extern long _fcreator;  // used in call to set output file creator type (see below) -- this is Mac specific!

    FILE *ifp, *ofp;
	double *xi, *temp ,*time;
	double r39[101] ,r40[101], tra40[101], tra39[101]; // dr39[101], 
	mediumarray age, sra40, sra39, scura, tilab, telab;
	double tmin[51],tpin[51];
	double E[11],D0[11],c[11];
	double b, c40, c39wd, dt; // average, 
	
	double slab;
	
	long imp, nsamp, j, np, ns, ni;
	short option; // fiddle

// following are from chist procedure of Lovera
	double snt, tem0;
	long ij, nt;
	
// following are from geosec procedure of Lovera
	double zit, avtemp, a1, uplus, cam, sum;
	double	*dzita, *d;
	long	n, jj, nj;
	long	madj, mfake,  m; //, min, upper_index;

// following are from lab procedure of Lovera
	double  kpie, prev_r40, change, zii, ab; //, zita_fudge;
	mediumarray zita; // ages; dr40;  rate;  NOT USED
	long jjj, k;
//	long kk, kx; NOT USED
	
// following are thresholds for some exponentials
	double ecam, elab, euplus, ichange;
  
// following are for heat schedule calculation
	double goal_loss[101], goal_age[101];
	long labsteps;
	long step, i;
	double target[101],cumDta2[21],guess[4],Arloss[4];
	double Dta2, a_term, volume, delta;
	double dummy;


	xi = (double *) calloc(50003, sizeof(double));
	temp = (double *) calloc(1006, sizeof(double));
	time = (double *) calloc(1006, sizeof(double));
	dzita = (double *) calloc(1006, sizeof(double));
	d = (double *) calloc(1006, sizeof(double));

SIOUXSettings.rows = 50;      // make output window a little bigger
SIOUXSettings.columns = 90;

SIOUXSettings.toppixel = 60;      // push it to upper left
SIOUXSettings.leftpixel = 200;

SIOUXSettings.autocloseonquit = 0;      // keep window up when Arvert done, but eliminate nag on quit about
SIOUXSettings.asktosaveonclose = 0;     // saving window contents
SIOUXSettings.showstatusline = 0;

SIOUXSetTitle("\pSimpleLovera 1.1.0");

	printf("        Program SimpleLovera 1.1.1\n");
	printf("\n");
	printf("\n");
	printf("Options\n");
	printf("    0 -- Proceed...\n");
	printf("    1 -- Quit: Head for the hills...\n");
	printf("---> ");

  	scanf("%hd", &option);
	
	if (option == 1)
	{
		exit(0);
	}
	
	ecam = 50;
	elab = 60;
	euplus = 50;

//  ------------ Get thermal history, diffusion parameters from file <lovera.in> -------------

	ifp = fopen("lovera.in", "r");
	
	fscanf(ifp,"%lf", &dt);   // temperature increment for discretization
//	assert((dt <= 20.0) && (dt >= 1.0));


	fscanf(ifp,"%lf", &ichange);   // convergence criterion for infinite series
//	assert((ichange <= 0.001) && (ichange >= 1.0e-9));


	fscanf(ifp,"%d", &imp);   // diffusion geometry 1=sphere  2=slab
	if (imp == 1) 
	{
			b = 6.0;    // spheres
			slab = 1.0;
	}
	else 
	{
			b = 8.0;    // slabs
			slab = 4.0;
	}
	
	fscanf(ifp, "%d", &np);   // read the number of time-temperature points		

	for (j = 1; j <= np; j++)
	{
		fscanf(ifp, "%lf%lf", &tmin[j], &tpin[j]);
//		printf("%d  %f  %f\n", j, tmin[j], tpin[j]);
		tpin[j] = tpin[j] + 273.15;  // geologic temperatures to Kelvin
	}
	fscanf(ifp, "%d", &nsamp);   // number of domains
	for (j = 1; j <= nsamp; j++)
	{
		fscanf(ifp, "%lf%lf%lf", &E[j], &D0[j], &c[j]);
//		printf("%d  %f  %f  %f\n", j, E[j], D0[j], c[j]);
		D0[j] = exp(2.3025851 * D0[j]);  // convert log10 D0 to true values
		D0[j] = D0[j] * 31557600e6;    // 1/sec to 1/m.y.
	}

	fclose(ifp);

//  ------------ Get losses and ages from file <goal.in> -------------

	ifp = fopen("goal.in", "r");

	fscanf(ifp, "%d", &ni);   // number of lab heating steps

	for (j = 1; j <= ni; j++)
	{
		fscanf(ifp, "%lf%lf%lf%d", &goal_loss[j], &goal_age[j]), &dummy, &i;
	}
	fclose(ifp);

	labsteps = ni; // patch variables together for this dirty port

//  ----------- now make a heating schedule (use 10 min as heating, makes no difference!) ----

	for (i = 1; i <= 10; i++)
	{
		cumDta2[i] = 0.0;
	}

	for (i = 1; i <= labsteps; i++)
	{
		target[i] = goal_loss[i];
//		assert((target[i] <= 1.0) && (target[i] != target[i-1]));
		if (target[i] == 1.0) //   protect against case of total loss
		{
			target[i] = 0.9999;
		}
	}

	delta = 1.0;
	guess[1] = 10; // these guesses MUST bracket real temperature value for first step!! 
	guess[2] = 3000;
	do                    //  deal with first step
	{
		guess[3] = (guess[1] + guess[2])/2;
		for (i = 1; i <= 3; i++) 
		{
			Arloss[i] = 0.0;
			volume = 0;
			for (j = 1; j <= nsamp; j++)
			{
				volume = volume + c[j];
				a_term = D0[j]/31557600e6 * 600;  // Do/a2 back to 1/sec, then times 10 min to get Dot/a2
				cumDta2[j] = a_term * exp(-E[j] / R / (guess[i] + 273.15));
				if (imp == 1)
				{
					if (cumDta2[j] <= 0.000001)
					{
						Arloss[i] = Arloss[i] + c[j]*(6 * sqrt(cumDta2[j]/pi));
					}
					else
					{
						long index;
						double sumseries, previous, change;
						sumseries = 0.0;
						previous = 1.0;
						index = 0;
						do
						{
							index = index + 1;
							change = fabs(previous - sumseries)/previous;
							previous = sumseries;
							sumseries = sumseries + 6/pi/pi/index/index*exp(-index*index*pi*pi*cumDta2[j]);
						}
						while (change > 0.000000001);
						Arloss[i] = Arloss[i] + c[j]*(1 - sumseries);
					}
				}
				else
				{
					if (cumDta2[j] < 0.159)
					{
						Arloss[i] = Arloss[i] + c[j]*(1.12838 * sqrt(cumDta2[j]));
						
					}
					else
					{
						Arloss[i] = Arloss[i] + c[j]*(1 - 0.810569 * exp(-2.467401 * cumDta2[j]));
					}
				}  // end if-imp
			}   // end for-j to domains					
			
			Arloss[i] = Arloss[i]/volume;

		} // for loop calculating all guess temperatures 
		
		if (Arloss[3] <= target[1])
		{
			guess[1] = guess[3];
		}
		else
		{
			guess[2] = guess[3];
		}
		delta = fabs((Arloss[3] - target[1]))/target[1];
	}
	while (delta > 0.00001);  // end conditional loop iterating guess temperatures 
	
	telab[1] = guess[3] + 273.15;
	
	for (j = 1; j <= nsamp; j++)
	{
		a_term = D0[j]/31557600e6 * 600;   // Dt/a2 for ten minutes
		cumDta2[j] = a_term * exp(-E[j] / R / (guess[3] + 273.15));
	}
		
	for (step = 2; step <= labsteps; step++)   /// now deal with all other steps
	{
		delta = 1.0;
		guess[1] = 10;  // for the record, guess temps work in C
		guess[2] = 3000;
		do
		{
			guess[3] = (guess[1] + guess[2])/2;
			for (i = 1; i <= 3; i++)
			{
				Arloss[i] = 0.0;
				for (j = 1; j <= nsamp; j++)
				{
					a_term = D0[j]/31557600e6 * 600;  // we're using 10 minutes duration
					Dta2 = cumDta2[j] + a_term * exp(-E[j] / R / (guess[i] + 273.15));
				if (imp == 1)
				{
					if (Dta2 <= 0.000001)
					{
						Arloss[i] = Arloss[i] + c[j]*(6 * sqrt(Dta2/pi));
					}
					else
					{
						long index;
						double sumseries, previous, change;
						sumseries = 0.0;
						previous = 1.0;
						index = 0;
						do
						{
							index = index + 1;
							change = fabs(previous - sumseries)/previous;
							previous = sumseries;
							sumseries = sumseries + 6/pi/pi/index/index*exp(-index*index*pi*pi*Dta2);
						}
						while (change > 0.000000001);
						Arloss[i] = Arloss[i] + c[j]*(1 - sumseries);
					}
				}
				else
					{
						if (Dta2 < 0.159)
						{
							Arloss[i] = Arloss[i] + c[j]*(1.12838 * sqrt(Dta2));
						}
						else
						{
							Arloss[i] = Arloss[i] + c[j]*(1 - 0.810569 * exp(-2.46740 * Dta2));
						}
					}
				}
				Arloss[i] = Arloss[i]/volume;
			} //  end loop through guess temperatures
			
			if (Arloss[3] <= target[step])
			{
				guess[1] = guess[3];
			}
				else
			{
				guess[2] = guess[3];
			}
			delta = fabs(Arloss[3] - target[step])/target[step];
		}
		while (delta > 0.001);
	
		telab[step] = guess[3] + 273.15;  // convert degrees C to K
		for (j = 1; j <= nsamp; j++)
		{
			a_term = D0[j]/31557600e6 * 600;   // Dt/a2 for ten minutes
			cumDta2[j] = cumDta2[j] + a_term * exp(-E[j] / R / (guess[3] + 273.15));
		}
	}  

	for (i = 1; i <= labsteps; i++)   //  load up the lab time array
	{
		tilab[i] = 10/5.256e11;  // minutes --> m.y.
	}

//  ---------- now begin actual Lovera algorithm ---------------------------------------------
	c40 = 1.0 - exp(-xlambd * tmin[1]);
	c39wd = exp(-xlambd * tmin[1]);
	
	printf("\n");
	printf("\n");
	printf("Finished reading what appears to be good input....\n");

	ns = 0;
	temp[1] = tpin[1];
	time[1] = tmin[1];
	tem0 = tpin[1];

// discretize the input thermal history
 
	for (k = 1; k <= np - 1; k++)
	{
		nt = (long) (floor(fabs(tpin[k] - tpin[k + 1]) / dt)) + ns;
		snt = tpin[k] - tpin[k + 1];
		if (fabs(snt) >= dt)
		{
			snt = snt / fabs(tpin[k] - tpin[k + 1]);
			for (ij = ns + 2; ij <= nt + 1; ij++)
			{
				temp[ij] = tem0 - snt * dt;
				time[ij] = InterpolateAge(temp[ij], tmin[k], tmin[k + 1], tpin[k], tpin[k + 1]);
				tem0 = temp[ij];
			}
			ns = nt;
			if ((long) floor(tpin[k] - tpin[k + 1]) % (long) floor(dt) != 0) // for C, had to use floor instead of TRUNC
			{
				temp[nt + 2] = tpin[k + 1];
				time[nt + 2] = tmin[k + 1];
				tem0 = temp[nt + 2];
				ns = ns + 1;
			}
			if (ns > nch - 2)
			{
				printf(" Program aborted: more than 1000 time steps\n");
				exit;
			}
		}
		else
		{
			temp[nt + 2] = tpin[k + 1];
			time[nt + 2] = tmin[k + 1];
			ns = ns + 1;
			tem0 = tpin[k + 1];	
		}
	}  // end of chist loop
	
/*	for (ij = 1; ij <= ns+1; ij++)   DEBUG -- Print out discretized time-temps
	{
					printf(" %d  %f  %f\n",ij,temp[ij],time[ij]);
	}
*/
	
	printf("Finished making discretizing what appears to be a legal temperature history....\n");
	printf("\n");
	
	for (j = 0; j <= ni; j++) // added this-- Lovera's code did not initialize  arrays to zero for summation }
	{
		tra40[j] = 0.0;
		tra39[j] = 0.0;
	}
	

// -------------------------- MAIN LOOP ------------------------------------

	for (j = 1; j <= nsamp; j++)   // calculate the age spectrum for each domain
	{
	//	printf("Starting to process domain: %2d\n",j);
		
    // following code is Lovera's GEOSEC procedure
		zit = 0.0;
		for (jj = 1; jj <= ns+1; jj++)
		{
			avtemp = (temp[jj + 1] + temp[jj]) / 2;
			d[jj] = D0[j]/slab * exp(-E[j] / R / avtemp);
		}

		dzita[ns + 1] = 0.0;

		for (nj = 1; nj <= ns; nj++)
		{
			jj = nj - 1;
			dzita[ns - jj] = dzita[ns + 1 - jj] + fabs(time[ns + 1 - jj] - time[ns - jj]) * d[ns - jj];
		}

// Computation of HXI(M)

		for (m = 0; m <= 49; m++)
		{
			sum = 0.0;
			for (n = 1; n <= ns; n++)
			{
				a1 = pow(((m*imp + 1) * pi),2.0) * d[n] - xlambd; 
				uplus = pow(((m*imp + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus) // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
						cam = 1.0;
					}
					else
					{
						cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}       // if-a1 conditional
					sum = sum + d[n] * pow(((m*imp + 1) * pi),2) / a1 * cam * exp(-uplus);
				}       // end of if-uplus conditional
			}        // end for n loop
			xi[m+1] = sum;
		}        // end of for m loop

		for (m = 1; m <= 310; m++)
		{
			mfake = m*5 + 49;
			sum = 0.0;
			for (n = 1; n <= ns; n++)
			{
				a1 = pow(((mfake*imp + 1) * pi),2.0) * d[n] - xlambd;
				uplus = pow(((mfake*imp + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus)  // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
							cam = 1.0;
					}
					else
					{
							cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}     // if-a1 conditional
					sum = sum + d[n] * pow(((mfake*imp + 1) * pi),2.0) / a1 * cam * exp(-uplus);
				}     // if-uplus conditional
			}      // end for n loop
			xi[mfake+1] = sum;
			for (madj = 1;madj <= 4;madj++)
			{
				xi[mfake+1-madj] = xi[mfake+1] - (xi[mfake+1] - xi[mfake+1-5])/5*madj;
			}
		}         // end  for m loop
				
		for (m = 1; m <= 242; m++)
		{
			mfake = m*200 + 1599;
			sum = 0.0;
			for (n = 1; n <= ns; n++)
			{
				a1 = pow(((mfake*imp + 1) * pi),2.0) * d[n] - xlambd;
				uplus = pow(((mfake*imp + 1) * pi),2.0) * dzita[n + 1] + xlambd * (time[1] - time[n + 1]);
				if (uplus <= euplus) // was 50
				{
					if ((a1 * fabs(time[n + 1] - time[n])) > ecam)
					{
						cam = 1.0;
					}
					else
					{
						cam = 1.0 - exp(-a1 * fabs(time[n + 1] - time[n]));
					}       //  end if-a1 conditional
					sum = sum + d[n] * pow(((mfake*imp + 1) * pi),2.0) / a1 * cam * exp(-uplus);
				}       //  end if-uplus conditional
			}        //  end for n loop
			xi[mfake+1] = sum;
			for (madj = 1;madj <= 199;madj++)
			{
				xi[mfake+1-madj] = xi[mfake+1] - (xi[mfake+1] - xi[mfake+1-200])/200*madj;
			}
		}   // end  for m loop

//  following code is Lovera's LAB procedure

		zii= 0.0;			

		for (nt = 1;nt <= ni;nt++)
		{
			zita[nt] = zii + D0[j]/slab * tilab[nt] * exp(-E[j] / R / telab[nt]);
			zii = zita[nt];
		}

		for (jjj = 1;jjj <= ni;jjj++)
		{
			r40[jjj] = 0.0;
			r39[jjj] = 0.0;
			k = -1;
			change = 1.0;
			prev_r40 = 0.0;
//    for k := 0 to nxi - 1 do  THIS WAS ORIGINAL
			while ((change > ichange) && (k < 49999))   // loop only until values stop changing
			{
				k = k + 1;
				kpie = pow(((k*imp + 1) * pi),2.0);
				if (zita[jjj] * kpie > elab)    //    was 100
				{
					ab = 0.0;
				}
				else
				{
					ab = exp(-zita[jjj] * kpie) / kpie;
					r39[jjj] = r39[jjj]  + ab;
				}
				r40[jjj] = r40[jjj] + (c40 - 1 + xi[k+1]) * (1.0 / kpie - ab);
				change = (r40[jjj] - prev_r40) / r40[jjj];
				prev_r40 = r40[jjj];
			}
				r40[jjj] = b * r40[jjj];
				r39[jjj] = 1.0 - b*r39[jjj];
		}

		for (k = 1; k <= ni;k++)
		{
			tra40[k] = c[j] * r40[k] + tra40[k];
			tra39[k] = c[j] * r39[k] + tra39[k];
		}
	}   //  of main loop in j

	for (j = 1; j <= ni;j++)      // final data reduction }
	{
		sra40[j] = tra40[j] - tra40[j - 1];
		sra39[j] = tra39[j] - tra39[j - 1];
		if ((sra40[j] <= 0.0) || (sra39[j] <= 0.0))
		{
			age[j] = 0.0;
		}
		else
		{
			scura[j] = sra40[j] / sra39[j];
			age[j] = log(1.0 + scura[j] / c39wd) / xlambd;
		}
	}

		_fcreator = 'XCEL'; // make the output text files have Excel as their creator--Mac OS only! PORT-ISSUE
	// if you want these text files to be opened with BBEdit, substitute 'R*ch' for 'XCEL' (case-sensitive!)

	ifp = fopen("Lovera.spec", "w");  // complete spectra

	fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", 0.0, 0.0, age[1]);   // we are writing out loss, goal_age, model age, and assuming loss calculated correctly!

	for (j = 1; j <= ni - 1;j++)
	{
		fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[j], goal_age[j], age[j]);  // spectrum format
		fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[j], goal_age[j + 1], age[j + 1]);
	}
	fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[ni], goal_age[ni], age[ni]);

	fclose(ifp);

	_fcreator = 'R*ch'; 
	
	ofp = fopen("Lovera.out", "w");        // just table of losses and ages
	for (j = 1; j <= ni - 1;j++)
	{
		fprintf(ofp,"%6.4f\t%7.2f\t%7.2f\n", tra39[j], goal_age[j], age[j]);  // simple table
		printf("%d\t%6.4f\t%7.2f\t%7.2f\n", j, tra39[j], goal_age[j], age[j]);

		fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[j], goal_age[j], age[j]);  // spectrum format
		fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[j], goal_age[j + 1], age[j + 1]);
	}

	fprintf(ifp,"%6.4f\t%7.2f\t%7.2f\n", tra39[ni], goal_age[ni], age[ni]);

	fprintf(ofp,"%6.4f\t%7.2f\t%7.2f\n", tra39[ni], goal_age[ni], age[ni]);
	printf("%d\t%6.4f\t%7.2f\t%7.2f\n", ni,tra39[ni], goal_age[ni], age[ni]);
	fclose(ofp);

	
	dummy = tra39[1]*age[1];
	for (j = 2; j <= ni - 1;j++)     // get integrated age of sample (integrated via ages, not Ar content)
	{
		dummy = dummy + (tra39[j]-tra39[j-1])*age[j];
	}

	printf("\nIntegrated model age is %6.2f Ma\n\n", dummy);  // print out integrated age

	printf("\n");
	printf("********* SimpleLovera finished with no worries! **********\n");

	
	return;
}               // ********* END OF main() ***********


// ---------------- Begin Definition of Functions -------------------

// -------------- Function interp_age --------------------

	double InterpolateAge (double y, double a1, double a2, double t1, double t2)
	{
		double interpAge;
		interpAge = a1 + (y - t1) * (a2 - a1) / (t2 - t1);
		return interpAge;
	}

// ---------------- End Definition of Functions ------------------- }